@@ -372,7 +372,13 @@ func (service *serviceType) handleIncomingPacket(bytes []byte) {
 						options.Host = value.(string)
 					}
 					if value, ok := request["port"]; ok {
-						options.Port = uint16(value.(int))
+						if value == 0 {
+							// 0 is the default value in Go, which we interpret as "try to
+							// pick port 8000". So Go uses -1 as the sentinel value instead.
+							options.Port = -1
+						} else {
+							options.Port = value.(int)
+						}
 					}
 					if value, ok := request["servedir"]; ok {
 						options.Servedir = value.(string)
@@ -418,11 +424,15 @@ func (service *serviceType) handleIncomingPacket(bytes []byte) {
 					if result, err := ctx.Serve(options); err != nil {
 						service.sendPacket(encodeErrorPacket(p.id, err))
 					} else {
+						hosts := make([]interface{}, len(result.Hosts))
+						for i, host := range result.Hosts {
+							hosts[i] = host
+						}
 						service.sendPacket(encodePacket(packet{
 							id: p.id,
 							value: map[string]interface{}{
-								"port": int(result.Port),
-								"host": result.Host,
+								"port":  int(result.Port),
+								"hosts": hosts,
 							},
 						}))
 					}
@@ -836,7 +846,6 @@ func (service *serviceType) convertPlugins(key int, jsPlugins interface{}, activ
 
 	var onResolveCallbacks []filteredCallback
 	var onLoadCallbacks []filteredCallback
-	hasOnStart := false
 	hasOnEnd := false
 
 	filteredCallbacks := func(pluginName string, kind string, items []interface{}) (result []filteredCallback, err error) {
@@ -860,10 +869,6 @@ func (service *serviceType) convertPlugins(key int, jsPlugins interface{}, activ
 		p := p.(map[string]interface{})
 		pluginName := p["name"].(string)
 
-		if p["onStart"].(bool) {
-			hasOnStart = true
-		}
-
 		if p["onEnd"].(bool) {
 			hasOnEnd = true
 		}
@@ -944,22 +949,20 @@ func (service *serviceType) convertPlugins(key int, jsPlugins interface{}, activ
 			}
 			activeBuild.mutex.Unlock()
 
-			// Only register "OnStart" if needed
-			if hasOnStart {
-				build.OnStart(func() (api.OnStartResult, error) {
-					response, ok := service.sendRequest(map[string]interface{}{
-						"command": "on-start",
-						"key":     key,
-					}).(map[string]interface{})
-					if !ok {
-						return api.OnStartResult{}, errors.New("The service was stopped")
-					}
-					return api.OnStartResult{
-						Errors:   decodeMessages(response["errors"].([]interface{})),
-						Warnings: decodeMessages(response["warnings"].([]interface{})),
-					}, nil
-				})
-			}
+			// Always register "OnStart" to clear "pluginData"
+			build.OnStart(func() (api.OnStartResult, error) {
+				response, ok := service.sendRequest(map[string]interface{}{
+					"command": "on-start",
+					"key":     key,
+				}).(map[string]interface{})
+				if !ok {
+					return api.OnStartResult{}, errors.New("The service was stopped")
+				}
+				return api.OnStartResult{
+					Errors:   decodeMessages(response["errors"].([]interface{})),
+					Warnings: decodeMessages(response["warnings"].([]interface{})),
+				}, nil
+			})
 
 			// Only register "OnResolve" if needed
 			if len(onResolveCallbacks) > 0 {
