@@ -100,7 +100,7 @@ func TestLowerExponentiationOperatorNoBundle(t *testing.T) {
 			UnsupportedJSFeatures: es(2015),
 			AbsOutputFile:         "/out.js",
 		},
-		expectedScanLog: `entry.js: ERROR: Big integer literals are not available in the configured target environment
+		expectedScanLog: `entry.js: WARNING: Big integer literals are not available in the configured target environment and may crash at run-time
 `,
 	})
 }
@@ -384,10 +384,10 @@ func TestTSLowerPrivateFieldAndMethodAvoidNameCollision2015(t *testing.T) {
 	lower_suite.expectBundled(t, bundled{
 		files: map[string]string{
 			"/entry.ts": `
-				class WeakMap {
+				export class WeakMap {
 					#x
 				}
-				class WeakSet {
+				export class WeakSet {
 					#y() {}
 				}
 			`,
@@ -405,7 +405,7 @@ func TestLowerPrivateGetterSetter2015(t *testing.T) {
 	lower_suite.expectBundled(t, bundled{
 		files: map[string]string{
 			"/entry.js": `
-				class Foo {
+				export class Foo {
 					get #foo() { return this.foo }
 					set #bar(val) { this.bar = val }
 					get #prop() { return this.prop }
@@ -456,7 +456,7 @@ func TestLowerPrivateGetterSetter2019(t *testing.T) {
 	lower_suite.expectBundled(t, bundled{
 		files: map[string]string{
 			"/entry.js": `
-				class Foo {
+				export class Foo {
 					get #foo() { return this.foo }
 					set #bar(val) { this.bar = val }
 					get #prop() { return this.prop }
@@ -507,7 +507,7 @@ func TestLowerPrivateGetterSetter2020(t *testing.T) {
 	lower_suite.expectBundled(t, bundled{
 		files: map[string]string{
 			"/entry.js": `
-				class Foo {
+				export class Foo {
 					get #foo() { return this.foo }
 					set #bar(val) { this.bar = val }
 					get #prop() { return this.prop }
@@ -608,7 +608,7 @@ func TestLowerPrivateMethod2019(t *testing.T) {
 	lower_suite.expectBundled(t, bundled{
 		files: map[string]string{
 			"/entry.js": `
-				class Foo {
+				export class Foo {
 					#field
 					#method() {}
 					baseline() {
@@ -650,7 +650,7 @@ func TestLowerPrivateMethod2020(t *testing.T) {
 	lower_suite.expectBundled(t, bundled{
 		files: map[string]string{
 			"/entry.js": `
-				class Foo {
+				export class Foo {
 					#field
 					#method() {}
 					baseline() {
@@ -757,7 +757,7 @@ func TestLowerPrivateMethodWithModifiers2020(t *testing.T) {
 	lower_suite.expectBundled(t, bundled{
 		files: map[string]string{
 			"/entry.js": `
-				class Foo {
+				export class Foo {
 					*#g() {}
 					async #a() {}
 					async *#ag() {}
@@ -1634,6 +1634,11 @@ func TestTSLowerClassField2020NoBundle(t *testing.T) {
 					static s_bar
 				}
 			`,
+			"/tsconfig.json": `{
+				"compilerOptions": {
+					"useDefineForClassFields": false
+				}
+			}`,
 		},
 		entryPaths: []string{"/entry.ts"},
 		options: config.Options{
@@ -1658,6 +1663,11 @@ func TestTSLowerClassPrivateFieldNextNoBundle(t *testing.T) {
 					static s_bar
 				}
 			`,
+			"/tsconfig.json": `{
+				"compilerOptions": {
+					"useDefineForClassFields": false
+				}
+			}`,
 		},
 		entryPaths: []string{"/entry.ts"},
 		options: config.Options{
@@ -2208,8 +2218,9 @@ func TestLowerPrivateClassFieldStaticIssue1424(t *testing.T) {
 		},
 		entryPaths: []string{"/entry.js"},
 		options: config.Options{
-			Mode:          config.ModeBundle,
-			AbsOutputFile: "/out.js",
+			Mode:                  config.ModeBundle,
+			AbsOutputFile:         "/out.js",
+			UnsupportedJSFeatures: compat.ClassPrivateMethod,
 		},
 	})
 }
@@ -2326,6 +2337,8 @@ func TestLowerForAwait2017(t *testing.T) {
 					async () => { for await (x.y of y) z(x) },
 					async () => { for await (let x of y) z(x) },
 					async () => { for await (const x of y) z(x) },
+					async () => { label: for await (const x of y) break label },
+					async () => { label: for await (const x of y) continue label },
 				]
 			`,
 		},
@@ -2347,6 +2360,8 @@ func TestLowerForAwait2015(t *testing.T) {
 					async () => { for await (x.y of y) z(x) },
 					async () => { for await (let x of y) z(x) },
 					async () => { for await (const x of y) z(x) },
+					async () => { label: for await (const x of y) break label },
+					async () => { label: for await (const x of y) continue label },
 				]
 			`,
 		},
@@ -2387,3 +2402,795 @@ func TestLowerNestedFunctionDirectEval(t *testing.T) {
 		},
 	})
 }
+
+func TestJavaScriptDecoratorsESNext(t *testing.T) {
+	lower_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				@x.y()
+				@(new y.x)
+				export default class Foo {
+					@x @y mUndef
+					@x @y mDef = 1
+					@x @y method() { return new Foo }
+					@x @y static sUndef
+					@x @y static sDef = new Foo
+					@x @y static sMethod() { return new Foo }
+				}
+			`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode:          config.ModePassThrough,
+			AbsOutputFile: "/out.js",
+		},
+	})
+}
+
+func TestJavaScriptAutoAccessorESNext(t *testing.T) {
+	lower_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/js-define.js": `
+				class Foo {
+					accessor one = 1
+					accessor #two = 2
+					accessor [three()] = 3
+
+					static accessor four = 4
+					static accessor #five = 5
+					static accessor [six()] = 6
+				}
+			`,
+			"/ts-define/ts-define.ts": `
+				class Foo {
+					accessor one = 1
+					accessor #two = 2
+					accessor [three()] = 3
+
+					static accessor four = 4
+					static accessor #five = 5
+					static accessor [six()] = 6
+				}
+				class Normal { accessor a = b; c = d }
+				class Private { accessor #a = b; c = d }
+				class StaticNormal { static accessor a = b; static c = d }
+				class StaticPrivate { static accessor #a = b; static c = d }
+			`,
+			"/ts-define/tsconfig.json": `{
+				"compilerOptions": {
+					"useDefineForClassFields": true,
+				},
+			}`,
+			"/ts-assign/ts-assign.ts": `
+				class Foo {
+					accessor one = 1
+					accessor #two = 2
+					accessor [three()] = 3
+
+					static accessor four = 4
+					static accessor #five = 5
+					static accessor [six()] = 6
+				}
+				class Normal { accessor a = b; c = d }
+				class Private { accessor #a = b; c = d }
+				class StaticNormal { static accessor a = b; static c = d }
+				class StaticPrivate { static accessor #a = b; static c = d }
+			`,
+			"/ts-assign/tsconfig.json": `{
+				"compilerOptions": {
+					"useDefineForClassFields": false,
+				},
+			}`,
+		},
+		entryPaths: []string{
+			"/js-define.js",
+			"/ts-define/ts-define.ts",
+			"/ts-assign/ts-assign.ts",
+		},
+		options: config.Options{
+			Mode:         config.ModePassThrough,
+			AbsOutputDir: "/out",
+		},
+	})
+}
+
+func TestJavaScriptAutoAccessorES2022(t *testing.T) {
+	lower_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/js-define.js": `
+				class Foo {
+					accessor one = 1
+					accessor #two = 2
+					accessor [three()] = 3
+
+					static accessor four = 4
+					static accessor #five = 5
+					static accessor [six()] = 6
+				}
+			`,
+			"/ts-define/ts-define.ts": `
+				class Foo {
+					accessor one = 1
+					accessor #two = 2
+					accessor [three()] = 3
+
+					static accessor four = 4
+					static accessor #five = 5
+					static accessor [six()] = 6
+				}
+				class Normal { accessor a = b; c = d }
+				class Private { accessor #a = b; c = d }
+				class StaticNormal { static accessor a = b; static c = d }
+				class StaticPrivate { static accessor #a = b; static c = d }
+			`,
+			"/ts-define/tsconfig.json": `{
+				"compilerOptions": {
+					"useDefineForClassFields": true,
+				},
+			}`,
+			"/ts-assign/ts-assign.ts": `
+				class Foo {
+					accessor one = 1
+					accessor #two = 2
+					accessor [three()] = 3
+
+					static accessor four = 4
+					static accessor #five = 5
+					static accessor [six()] = 6
+				}
+				class Normal { accessor a = b; c = d }
+				class Private { accessor #a = b; c = d }
+				class StaticNormal { static accessor a = b; static c = d }
+				class StaticPrivate { static accessor #a = b; static c = d }
+			`,
+			"/ts-assign/tsconfig.json": `{
+				"compilerOptions": {
+					"useDefineForClassFields": false,
+				},
+			}`,
+		},
+		entryPaths: []string{
+			"/js-define.js",
+			"/ts-define/ts-define.ts",
+			"/ts-assign/ts-assign.ts",
+		},
+		options: config.Options{
+			Mode:                  config.ModePassThrough,
+			AbsOutputDir:          "/out",
+			UnsupportedJSFeatures: es(2022),
+		},
+	})
+}
+
+func TestJavaScriptAutoAccessorES2021(t *testing.T) {
+	lower_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/js-define.js": `
+				class Foo {
+					accessor one = 1
+					accessor #two = 2
+					accessor [three()] = 3
+
+					static accessor four = 4
+					static accessor #five = 5
+					static accessor [six()] = 6
+				}
+			`,
+			"/ts-define/ts-define.ts": `
+				class Foo {
+					accessor one = 1
+					accessor #two = 2
+					accessor [three()] = 3
+
+					static accessor four = 4
+					static accessor #five = 5
+					static accessor [six()] = 6
+				}
+				class Normal { accessor a = b; c = d }
+				class Private { accessor #a = b; c = d }
+				class StaticNormal { static accessor a = b; static c = d }
+				class StaticPrivate { static accessor #a = b; static c = d }
+			`,
+			"/ts-define/tsconfig.json": `{
+				"compilerOptions": {
+					"useDefineForClassFields": true,
+				},
+			}`,
+			"/ts-assign/ts-assign.ts": `
+				class Foo {
+					accessor one = 1
+					accessor #two = 2
+					accessor [three()] = 3
+
+					static accessor four = 4
+					static accessor #five = 5
+					static accessor [six()] = 6
+				}
+				class Normal { accessor a = b; c = d }
+				class Private { accessor #a = b; c = d }
+				class StaticNormal { static accessor a = b; static c = d }
+				class StaticPrivate { static accessor #a = b; static c = d }
+			`,
+			"/ts-assign/tsconfig.json": `{
+				"compilerOptions": {
+					"useDefineForClassFields": false,
+				},
+			}`,
+		},
+		entryPaths: []string{
+			"/js-define.js",
+			"/ts-define/ts-define.ts",
+			"/ts-assign/ts-assign.ts",
+		},
+		options: config.Options{
+			Mode:                  config.ModePassThrough,
+			AbsOutputDir:          "/out",
+			UnsupportedJSFeatures: es(2021),
+		},
+	})
+}
+
+func TestLowerUsing(t *testing.T) {
+	lower_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				using a = b
+				await using c = d
+				if (nested) {
+					using x = 1
+					await using y = 2
+				}
+
+				function foo() {
+					using a = b
+					if (nested) {
+						using x = 1
+					}
+				}
+
+				async function bar() {
+					using a = b
+					await using c = d
+					if (nested) {
+						using x = 1
+						await using y = 2
+					}
+				}
+			`,
+			"/loops.js": `
+				for (using a of b) c(() => a)
+				for (await using d of e) f(() => d)
+				for await (using g of h) i(() => g)
+				for await (await using j of k) l(() => j)
+
+				if (nested) {
+					for (using a of b) c(() => a)
+					for (await using d of e) f(() => d)
+					for await (using g of h) i(() => g)
+					for await (await using j of k) l(() => j)
+				}
+
+				function foo() {
+					for (using a of b) c(() => a)
+				}
+
+				async function bar() {
+					for (using a of b) c(() => a)
+					for (await using d of e) f(() => d)
+					for await (using g of h) i(() => g)
+					for await (await using j of k) l(() => j)
+				}
+			`,
+			"/switch.js": `
+				using x = y
+				switch (foo) {
+					case 0: using c = d
+					default: using e = f
+				}
+				switch (foo) {
+					case 0: await using c = d
+					default: using e = f
+				}
+
+				async function foo() {
+					using x = y
+					switch (foo) {
+						case 0: using c = d
+						default: using e = f
+					}
+					switch (foo) {
+						case 0: await using c = d
+						default: using e = f
+					}
+				}
+			`,
+		},
+		entryPaths: []string{
+			"/entry.js",
+			"/loops.js",
+			"/switch.js",
+		},
+		options: config.Options{
+			Mode:                  config.ModePassThrough,
+			AbsOutputDir:          "/out",
+			UnsupportedJSFeatures: compat.Using,
+		},
+	})
+}
+
+func TestLowerUsingUnsupportedAsync(t *testing.T) {
+	lower_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				function foo() {
+					using a = b
+					if (nested) {
+						using x = 1
+					}
+				}
+
+				async function bar() {
+					using a = b
+					await using c = d
+					if (nested) {
+						using x = 1
+						await using y = 2
+					}
+				}
+			`,
+			"/loops.js": `
+				for (using a of b) c(() => a)
+
+				if (nested) {
+					for (using a of b) c(() => a)
+				}
+
+				function foo() {
+					for (using a of b) c(() => a)
+				}
+
+				async function bar() {
+					for (using a of b) c(() => a)
+					for (await using d of e) f(() => d)
+				}
+			`,
+			"/switch.js": `
+				using x = y
+				switch (foo) {
+					case 0: using c = d
+					default: using e = f
+				}
+
+				async function foo() {
+					using x = y
+					switch (foo) {
+						case 0: using c = d
+						default: using e = f
+					}
+					switch (foo) {
+						case 0: await using c = d
+						default: using e = f
+					}
+				}
+			`,
+		},
+		entryPaths: []string{
+			"/entry.js",
+			"/loops.js",
+			"/switch.js",
+		},
+		options: config.Options{
+			Mode:                  config.ModePassThrough,
+			AbsOutputDir:          "/out",
+			UnsupportedJSFeatures: compat.AsyncAwait | compat.TopLevelAwait,
+		},
+	})
+}
+
+func TestLowerUsingUnsupportedUsingAndAsync(t *testing.T) {
+	lower_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				function foo() {
+					using a = b
+					if (nested) {
+						using x = 1
+					}
+				}
+
+				async function bar() {
+					using a = b
+					await using c = d
+					if (nested) {
+						using x = 1
+						await using y = 2
+					}
+				}
+			`,
+			"/loops.js": `
+				for (using a of b) c(() => a)
+
+				if (nested) {
+					for (using a of b) c(() => a)
+				}
+
+				function foo() {
+					for (using a of b) c(() => a)
+				}
+
+				async function bar() {
+					for (using a of b) c(() => a)
+					for (await using d of e) f(() => d)
+				}
+			`,
+			"/switch.js": `
+				using x = y
+				switch (foo) {
+					case 0: using c = d
+					default: using e = f
+				}
+
+				async function foo() {
+					using x = y
+					switch (foo) {
+						case 0: using c = d
+						default: using e = f
+					}
+					switch (foo) {
+						case 0: await using c = d
+						default: using e = f
+					}
+				}
+			`,
+		},
+		entryPaths: []string{
+			"/entry.js",
+			"/loops.js",
+			"/switch.js",
+		},
+		options: config.Options{
+			Mode:                  config.ModePassThrough,
+			AbsOutputDir:          "/out",
+			UnsupportedJSFeatures: compat.Using | compat.AsyncAwait | compat.TopLevelAwait,
+		},
+	})
+}
+
+func TestLowerUsingHoisting(t *testing.T) {
+	lower_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/hoist-use-strict.js": `
+				"use strict"
+				using a = b
+				function foo() {
+					"use strict"
+					using a = b
+				}
+			`,
+			"/hoist-directive.js": `
+				"use wtf"
+				using a = b
+				function foo() {
+					"use wtf"
+					using a = b
+				}
+			`,
+			"/hoist-import.js": `
+				using a = b
+				import "./foo"
+				using c = d
+			`,
+			"/hoist-export-star.js": `
+				using a = b
+				export * from './foo'
+				using c = d
+			`,
+			"/hoist-export-from.js": `
+				using a = b
+				export {x, y} from './foo'
+				using c = d
+			`,
+			"/hoist-export-clause.js": `
+				using a = b
+				export {a, c as 'c!'}
+				using c = d
+			`,
+			"/hoist-export-local-direct.js": `
+				using a = b
+				export var ac1 = [a, c], { x: [x1] } = foo
+				export let a1 = a, { y: [y1] } = foo
+				export const c1 = c, { z: [z1] } = foo
+				var ac2 = [a, c], { x: [x2] } = foo
+				let a2 = a, { y: [y2] } = foo
+				const c2 = c, { z: [z2] } = foo
+				using c = d
+			`,
+			"/hoist-export-local-indirect.js": `
+				using a = b
+				var ac1 = [a, c], { x: [x1] } = foo
+				let a1 = a, { y: [y1] } = foo
+				const c1 = c, { z: [z1] } = foo
+				var ac2 = [a, c], { x: [x2] } = foo
+				let a2 = a, { y: [y2] } = foo
+				const c2 = c, { z: [z2] } = foo
+				using c = d
+				export {x1, y1, z1}
+			`,
+			"/hoist-export-class-direct.js": `
+				using a = b
+				export class Foo1 { ac = [a, c] }
+				export class Bar1 { ac = [a, c, Bar1] }
+				class Foo2 { ac = [a, c] }
+				class Bar2 { ac = [a, c, Bar2] }
+				using c = d
+			`,
+			"/hoist-export-class-indirect.js": `
+				using a = b
+				class Foo1 { ac = [a, c] }
+				class Bar1 { ac = [a, c, Bar1] }
+				class Foo2 { ac = [a, c] }
+				class Bar2 { ac = [a, c, Bar2] }
+				using c = d
+				export {Foo1, Bar1}
+			`,
+			"/hoist-export-function-direct.js": `
+				using a = b
+				export function foo1() { return [a, c] }
+				export function bar1() { return [a, c, bar1] }
+				function foo2() { return [a, c] }
+				function bar2() { return [a, c, bar2] }
+				using c = d
+			`,
+			"/hoist-export-function-indirect.js": `
+				using a = b
+				function foo1() { return [a, c] }
+				function bar1() { return [a, c, bar1] }
+				function foo2() { return [a, c] }
+				function bar2() { return [a, c, bar2] }
+				using c = d
+				export {foo1, bar1}
+			`,
+			"/hoist-export-default-class-name-unused.js": `
+				using a = b
+				export default class Foo {
+					ac = [a, c]
+				}
+				using c = d
+			`,
+			"/hoist-export-default-class-name-used.js": `
+				using a = b
+				export default class Foo {
+					ac = [a, c, Foo]
+				}
+				using c = d
+			`,
+			"/hoist-export-default-class-anonymous.js": `
+				using a = b
+				export default class {
+					ac = [a, c]
+				}
+				using c = d
+			`,
+			"/hoist-export-default-function-name-unused.js": `
+				using a = b
+				export default function foo() {
+					return [a, c]
+				}
+				using c = d
+			`,
+			"/hoist-export-default-function-name-used.js": `
+				using a = b
+				export default function foo() {
+					return [a, c, foo]
+				}
+				using c = d
+			`,
+			"/hoist-export-default-function-anonymous.js": `
+				using a = b
+				export default function() {
+					return [a, c]
+				}
+				using c = d
+			`,
+			"/hoist-export-default-expr.js": `
+				using a = b
+				export default [a, c]
+				using c = d
+			`,
+		},
+		entryPaths: []string{
+			"/hoist-use-strict.js",
+			"/hoist-directive.js",
+			"/hoist-import.js",
+			"/hoist-export-star.js",
+			"/hoist-export-from.js",
+			"/hoist-export-clause.js",
+			"/hoist-export-local-direct.js",
+			"/hoist-export-local-indirect.js",
+			"/hoist-export-class-direct.js",
+			"/hoist-export-class-indirect.js",
+			"/hoist-export-function-direct.js",
+			"/hoist-export-function-indirect.js",
+			"/hoist-export-default-class-name-unused.js",
+			"/hoist-export-default-class-name-used.js",
+			"/hoist-export-default-class-anonymous.js",
+			"/hoist-export-default-function-name-unused.js",
+			"/hoist-export-default-function-name-used.js",
+			"/hoist-export-default-function-anonymous.js",
+			"/hoist-export-default-expr.js",
+		},
+		options: config.Options{
+			Mode:                  config.ModePassThrough,
+			AbsOutputDir:          "/out",
+			UnsupportedJSFeatures: compat.Using,
+		},
+	})
+}
+
+func TestLowerUsingInsideTSNamespace(t *testing.T) {
+	lower_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.ts": `
+				namespace ns {
+					export let a = b
+					using c = d
+					export let e = f
+				}
+			`,
+		},
+		entryPaths: []string{"/entry.ts"},
+		options: config.Options{
+			Mode:                  config.ModePassThrough,
+			AbsOutputDir:          "/out",
+			UnsupportedJSFeatures: compat.Using,
+		},
+	})
+}
+
+func TestLowerAsyncGenerator(t *testing.T) {
+	lower_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.ts": `
+				async function* foo() {
+					yield
+					yield x
+					yield *x
+					await using x = await y
+					for await (let x of y) {}
+					for await (await using x of y) {}
+				}
+				foo = async function* () {
+					yield
+					yield x
+					yield *x
+					await using x = await y
+					for await (let x of y) {}
+					for await (await using x of y) {}
+				}
+				foo = { async *bar () {
+					yield
+					yield x
+					yield *x
+					await using x = await y
+					for await (let x of y) {}
+					for await (await using x of y) {}
+				} }
+				class Foo { async *bar () {
+					yield
+					yield x
+					yield *x
+					await using x = await y
+					for await (let x of y) {}
+					for await (await using x of y) {}
+				} }
+				Foo = class { async *bar () {
+					yield
+					yield x
+					yield *x
+					await using x = await y
+					for await (let x of y) {}
+					for await (await using x of y) {}
+				} }
+				async function bar() {
+					await using x = await y
+					for await (let x of y) {}
+					for await (await using x of y) {}
+				}
+			`,
+		},
+		entryPaths: []string{"/entry.ts"},
+		options: config.Options{
+			Mode:                  config.ModePassThrough,
+			AbsOutputDir:          "/out",
+			UnsupportedJSFeatures: compat.AsyncGenerator,
+		},
+	})
+}
+
+func TestLowerAsyncGeneratorNoAwait(t *testing.T) {
+	lower_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.ts": `
+				async function* foo() {
+					yield
+					yield x
+					yield *x
+					await using x = await y
+					for await (let x of y) {}
+					for await (await using x of y) {}
+				}
+				foo = async function* () {
+					yield
+					yield x
+					yield *x
+					await using x = await y
+					for await (let x of y) {}
+					for await (await using x of y) {}
+				}
+				foo = { async *bar () {
+					yield
+					yield x
+					yield *x
+					await using x = await y
+					for await (let x of y) {}
+					for await (await using x of y) {}
+				} }
+				class Foo { async *bar () {
+					yield
+					yield x
+					yield *x
+					await using x = await y
+					for await (let x of y) {}
+					for await (await using x of y) {}
+				} }
+				Foo = class { async *bar () {
+					yield
+					yield x
+					yield *x
+					await using x = await y
+					for await (let x of y) {}
+					for await (await using x of y) {}
+				} }
+				async function bar() {
+					await using x = await y
+					for await (let x of y) {}
+					for await (await using x of y) {}
+				}
+			`,
+		},
+		entryPaths: []string{"/entry.ts"},
+		options: config.Options{
+			Mode:                  config.ModePassThrough,
+			AbsOutputDir:          "/out",
+			UnsupportedJSFeatures: compat.AsyncGenerator | compat.AsyncAwait,
+		},
+	})
+}
+
+func TestJavaScriptDecoratorsBundleIssue3768(t *testing.T) {
+	lower_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/base-instance-method.js":   `class Foo { @dec foo() { return Foo } }`,
+			"/base-instance-field.js":    `class Foo { @dec foo = Foo }`,
+			"/base-instance-accessor.js": `class Foo { @dec accessor foo = Foo }`,
+
+			"/base-static-method.js":   `class Foo { @dec static foo() { return Foo } }`,
+			"/base-static-field.js":    `class Foo { @dec static foo = Foo }`,
+			"/base-static-accessor.js": `class Foo { @dec static accessor foo = Foo }`,
+
+			"/derived-instance-method.js":   `class Foo extends Bar { @dec foo() { return Foo } }`,
+			"/derived-instance-field.js":    `class Foo extends Bar { @dec foo = Foo }`,
+			"/derived-instance-accessor.js": `class Foo extends Bar { @dec accessor foo = Foo }`,
+
+			"/derived-static-method.js":   `class Foo extends Bar { @dec static foo() { return Foo } }`,
+			"/derived-static-field.js":    `class Foo extends Bar { @dec static foo = Foo }`,
+			"/derived-static-accessor.js": `class Foo extends Bar { @dec static accessor foo = Foo }`,
+		},
+		entryPaths: []string{"/*"},
+		options: config.Options{
+			Mode:                  config.ModeBundle,
+			AbsOutputDir:          "/out",
+			UnsupportedJSFeatures: compat.Decorators,
+		},
+	})
+}
