@@ -27,8 +27,8 @@ import (
 )
 
 func es(version int) compat.JSFeature {
-	return compat.UnsupportedJSFeatures(map[compat.Engine][]int{
-		compat.ES: {version},
+	return compat.UnsupportedJSFeatures(map[compat.Engine]compat.Semver{
+		compat.ES: {Parts: []int{version}},
 	})
 }
 
@@ -125,6 +125,9 @@ func (s *suite) __expectBundledImpl(t *testing.T, args bundled, fsKind fs.MockKi
 		if args.absWorkingDir == "" {
 			args.absWorkingDir = "/"
 		}
+		if args.options.AbsOutputDir == "" {
+			args.options.AbsOutputDir = args.absWorkingDir // Match the behavior of the API in this case
+		}
 
 		// Handle conversion to Windows-style paths
 		if fsKind == fs.MockWindows {
@@ -153,7 +156,7 @@ func (s *suite) __expectBundledImpl(t *testing.T, args bundled, fsKind fs.MockKi
 			args.options.AbsOutputFile = unix2win(args.options.AbsOutputFile)
 			args.options.AbsOutputBase = unix2win(args.options.AbsOutputBase)
 			args.options.AbsOutputDir = unix2win(args.options.AbsOutputDir)
-			args.options.TsConfigOverride = unix2win(args.options.TsConfigOverride)
+			args.options.TSConfigPath = unix2win(args.options.TSConfigPath)
 		}
 
 		// Run the bundler
@@ -161,7 +164,7 @@ func (s *suite) __expectBundledImpl(t *testing.T, args bundled, fsKind fs.MockKi
 		caches := cache.MakeCacheSet()
 		mockFS := fs.MockFS(args.files, fsKind, args.absWorkingDir)
 		args.options.OmitRuntimeForTests = true
-		bundle := bundler.ScanBundle(log, mockFS, caches, entryPoints, args.options, nil)
+		bundle := bundler.ScanBundle(config.BuildCall, log, mockFS, caches, entryPoints, args.options, nil)
 		msgs := log.Done()
 		assertLog(t, msgs, args.expectedScanLog)
 
